#!/usr/bin/env python
from numpy import *
from matplotlib.pyplot import *
from scipy.linalg import expm, solve
from ode45 import ode45


# ex c

def hphi1(J, z, h):
    """Input:
    J:  Matrix
    z:  Vektor
    h:  Skalar

    Output:  h * (e^(h*J) - I) * J^(-1) * z
    """
    result = z

    ############################################################
    #                                                          #
    # TODO: Implementieren Sie: h * (e^(h*J) - 1) * J^(-1) * z #
    #       Benutzen Sie *keine* explizite Matrixinverse.      #
    #                                                          #
    ############################################################

    return result


def h2phi2(J, z, h):
    """Input:
    J:  Matrix
    z:  Vektor
    h:  Skalar

    Output:  h^2 * (e^(h*J) - h*J - I) * J^(-2) * z
    """
    result = z

    ####################################################################
    #                                                                  #
    # TODO: Implementieren Sie: h^2 * (e^(h*J) - h*J - 1) * J^(-2) * z #
    #       Benutzen Sie *keine* explizite Matrixinverse.              #
    #                                                                  #
    ####################################################################

    return result


# ex d

def pope(nsteps, T0, T, y0, f, dfdy, dfdt):
    """
    Exponentielles Rosenbrock-Euler-Verfahren fuer
    ODEs mit nicht autonomer rechter Seite.

    Argumente:
    nsteps:  Anzahl Zeitschritte
    T0:      Anfangszeit
    T:       Endzeit
    y0:      Startwert zur Zeit t=T0
    f:       Rechte Seite y' = f(t, y)
    dfdy:    Jacobi-Matrix df/dy
    dfdt:    Zeitableitung df/dt

    Return Werte:
    t:       Zeitpunkte [T0, ..., T]
    y:       Loesung y(t)
    """
    t, h = linspace(T0, T, nsteps, retstep=True)

    # Speicherallokation
    y0 = atleast_1d(y0)
    y = zeros((nsteps, y0.shape[0]))

    ##################################################
    #                                                #
    # TODO: Implementieren Sie die Methode von Pope. #
    #                                                #
    ##################################################

    return t, y


# ex d

# Constants
ggg = 0.1
ell = 0.07
eps = 1e-2

###################################################################
#                                                                 #
# TODO: Berechnen Sie die rechte Seite sowie die zwei Ableitungen #
#                                                                 #
###################################################################

f = lambda t, u: array([0.0, 0.0])

# Jacobi-Matrix df/du
dfdu = lambda t, u: array([[0.0, 0.0],
                           [0.0, 0.0]])

# Time-dependence df/dt
dfdt = lambda t, u: array([0.0, 0.0])


# Anfangswert
T0 = 0.0
T = 10.0

u0 = array([ 0.0,
            -0.4])

nsteps = 2*10**4

t = zeros(nsteps)
u = zeros((nsteps, 2))

####################################################################
#                                                                  #
# TODO: Simulieren Sie die Gleichung mit der Methode von Pope      #
#       und berechnen Sie den maximalen Wert von 'theta'           #
#                                                                  #
####################################################################

maxamp = 0.0
print("Maximale Amplitude (Verfahren von Pope)")
print(maxamp)


figure()
plot(t, u[:,0], '-', label=r'$\Theta(t)$')
hlines(maxamp, T0, T, 'r')
hlines(-maxamp, T0, T, 'r')
grid(True)
xlim(T0, T)
ylim(-0.3, 0.3)
legend(loc='upper right', framealpha=0.8)
xlabel(r"Time $t$")
ylabel(r"Solution $\Theta(t)$")
savefig("ExpVer_pope.png")


# ex e

options = {'reltol':1e-6, 'abstol':1e-7, 'stats':'on'}

##########################################################
#                                                        #
# TODO: Simulieren Sie die Gleichung mit 'ode45'         #
#       und berechnen Sie den maximalen Wert von 'theta' #
#                                                        #
##########################################################

print("Maximale Amplitude (ode45)")
print(maxamp)

figure()
plot(t, u[:,0], '-', label=r'$\Theta(t)$')
hlines(maxamp, T0, T, 'r')
hlines(-maxamp, T0, T, 'r')
grid(True)
xlim(T0, T)
ylim(-0.3, 0.3)
legend(loc='upper right', framealpha=0.8)
xlabel(r"Time $t$")
ylabel(r"Solution $\Theta(t)$")
savefig("ExpVer_ode45.png")
